
#include <windows.h>  
#include "AnimPal.h" 


#if defined (WIN32)
	#define IS_WIN32 TRUE
#else
	#define IS_WIN32 FALSE
#endif

#define IS_NT      IS_WIN32 && (BOOL)(GetVersion() < 0x80000000)
#define IS_WIN32S  IS_WIN32 && (BOOL)(!(IS_NT) && (LOBYTE(LOWORD(GetVersion()))<4))
#define IS_WIN95   (BOOL)(!(IS_NT) && !(IS_WIN32S)) && IS_WIN32

HINSTANCE hInst;   // current instance

LPCTSTR lpszAppName  = "MyApp";
LPCTSTR lpszTitle    = "My Application"; 

BOOL RegisterWin95( CONST WNDCLASS* lpwc );

int APIENTRY WinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance,
                      LPTSTR lpCmdLine, int nCmdShow)
{
   MSG      msg;
   HWND     hWnd; 
   WNDCLASS wc;

   // Register the main application window class.
   //............................................
   wc.style         = CS_HREDRAW | CS_VREDRAW | CS_OWNDC;
   wc.lpfnWndProc   = (WNDPROC)WndProc;       
   wc.cbClsExtra    = 0;                      
   wc.cbWndExtra    = 0;                      
   wc.hInstance     = hInstance;              
   wc.hIcon         = LoadIcon( hInstance, lpszAppName ); 
   wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
   wc.hbrBackground = (HBRUSH)(COLOR_WINDOW+1);
   wc.lpszMenuName  = lpszAppName;              
   wc.lpszClassName = lpszAppName;              

   if ( IS_WIN95 )
   {
      if ( !RegisterWin95( &wc ) )
         return( FALSE );
   }
   else if ( !RegisterClass( &wc ) )
      return( FALSE );

   hInst = hInstance; 

   // Create the main application window.
   //....................................
   hWnd = CreateWindow( lpszAppName, 
                        lpszTitle,    
                        WS_OVERLAPPEDWINDOW, 
                        CW_USEDEFAULT, 0, 
                        CW_USEDEFAULT, 0,  
                        NULL,              
                        NULL,              
                        hInstance,         
                        NULL               
                      );

   if ( !hWnd ) 
      return( FALSE );

   ShowWindow( hWnd, nCmdShow ); 
   UpdateWindow( hWnd );         

   while( GetMessage( &msg, NULL, 0, 0) )   
   {
      TranslateMessage( &msg ); 
      DispatchMessage( &msg );  
   }

   return( msg.wParam ); 
}


BOOL RegisterWin95( CONST WNDCLASS* lpwc )
{
   WNDCLASSEX wcex;

   wcex.style         = lpwc->style;
   wcex.lpfnWndProc   = lpwc->lpfnWndProc;
   wcex.cbClsExtra    = lpwc->cbClsExtra;
   wcex.cbWndExtra    = lpwc->cbWndExtra;
   wcex.hInstance     = lpwc->hInstance;
   wcex.hIcon         = lpwc->hIcon;
   wcex.hCursor       = lpwc->hCursor;
   wcex.hbrBackground = lpwc->hbrBackground;
   wcex.lpszMenuName  = lpwc->lpszMenuName;
   wcex.lpszClassName = lpwc->lpszClassName;

   // Added elements for Windows 95.
   //...............................
   wcex.cbSize = sizeof(WNDCLASSEX);
   wcex.hIconSm = LoadImage(wcex.hInstance, lpwc->lpszClassName, 
                            IMAGE_ICON, 16, 16,
                            LR_DEFAULTCOLOR );
			
   return RegisterClassEx( &wcex );
}

#define NUMBERCOLORS 128

LRESULT CALLBACK WndProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
static LPLOGPALETTE pLogPal;
static HPALETTE     hPal;

   switch( uMsg )
   {
      case WM_CREATE :
              {
                 int i;

                 // Allocate space for the logical palette.
                 //........................................
                 pLogPal = HeapAlloc( GetProcessHeap(), HEAP_ZERO_MEMORY, 
                                      sizeof(LOGPALETTE) + 
                                      (NUMBERCOLORS * sizeof(PALETTEENTRY)) );

                 // Initialize the palette with 1 red color and the
                 // rest of the colors being white.
                 //................................................
                 pLogPal->palVersion    = 0x300;
                 pLogPal->palNumEntries = NUMBERCOLORS;

                 for( i = 0; i < NUMBERCOLORS; i++ )
                 {
                    pLogPal->palPalEntry[i].peRed   = 255;
                    pLogPal->palPalEntry[i].peGreen = (i==0 ? 0 : 255 );
                    pLogPal->palPalEntry[i].peBlue  = (i==0 ? 0 : 255 );
                    pLogPal->palPalEntry[i].peFlags = PC_RESERVED;
                 }

                 // Create the palette.
                 //....................
                 hPal = CreatePalette( pLogPal );
              }
              break;

      case WM_PAINT :
              {
                 PAINTSTRUCT ps;
                 HBRUSH      hBrush;
                 int         i;

                 // Begin painting and select and realize the palette.
                 //...................................................
                 BeginPaint( hWnd, &ps );
                 SelectPalette( ps.hdc, hPal, FALSE );
                 RealizePalette( ps.hdc );

                 // Select a null pen.
                 //...................
                 SelectObject( ps.hdc, GetStockObject( NULL_PEN ) );

                 // Paint a rectangle in each of the
                 // colors in our logical palette.
                 //.................................
                 for ( i = 0; i < NUMBERCOLORS; i++ )
                 {
                    // Create a brush of the palette color and select it.
                    //...................................................
                    hBrush = CreateSolidBrush( PALETTEINDEX(i) );
                    SelectObject( ps.hdc, hBrush );

                    // Paint the rectangle.
                    //.....................
                    Rectangle( ps.hdc, i*4, 0, (i*4)+25, 50 );

                    // Select the stock white brush object.
                    //.....................................
                    SelectObject( ps.hdc, GetStockObject( WHITE_BRUSH ) );

                    // Delete the brush we created.
                    //.............................
                    DeleteObject( hBrush );
                 }

                 // End painting.
                 EndPaint( hWnd, &ps );
              }
              break;

      case WM_COMMAND :
              switch( LOWORD( wParam ) )
              {
                 case IDM_TEST :
                        {
                           int i;
                            
                           // Cycle through each of the palette entries
                           // changing on entry at a time to red and then
                           // calling animate palette to show the change.
                           //............................................
                           for ( i = 0; i < NUMBERCOLORS; i++ )
                           {
                              pLogPal->palPalEntry[i].peGreen = 255;
                              pLogPal->palPalEntry[i].peBlue  = 255;
                              if ( i < NUMBERCOLORS-1 )
                              {
                                 pLogPal->palPalEntry[i+1].peGreen = 0;
                                 pLogPal->palPalEntry[i+1].peBlue  = 0;
                              }
                              else
                              {
                                 pLogPal->palPalEntry[0].peGreen = 0;
                                 pLogPal->palPalEntry[0].peBlue  = 0;
                              }

                              AnimatePalette( hPal, 0, NUMBERCOLORS, 
                                              pLogPal->palPalEntry );
                           }
                        }
                        break;

                 case IDM_ABOUT :
                        DialogBox( hInst, "AboutBox", hWnd, (DLGPROC)About );
                        break;

                 case IDM_EXIT :
                        DestroyWindow( hWnd );
                        break;
              }
              break;
      
      case WM_DESTROY :
              // Free the memory for the logical palette
              // and delete the palette object handle.
              //........................................
              HeapFree( GetProcessHeap(), 0, pLogPal );
              DeleteObject( hPal );

              PostQuitMessage(0);
              break;

      default :
            return( DefWindowProc( hWnd, uMsg, wParam, lParam ) );
   }

   return( 0L );
}


LRESULT CALLBACK About( HWND hDlg,           
                        UINT message,        
                        WPARAM wParam,       
                        LPARAM lParam)
{
   switch (message) 
   {
       case WM_INITDIALOG: 
               return (TRUE);

       case WM_COMMAND:                              
               if (   LOWORD(wParam) == IDOK         
                   || LOWORD(wParam) == IDCANCEL)    
               {
                       EndDialog(hDlg, TRUE);        
                       return (TRUE);
               }
               break;
   }

   return (FALSE); 
}
